	INCLUDE EFR32MG12.inc			; CPU register definitions
	PRESERVE8
	THUMB					; use thumb instruction set
		
	AREA    |.text|, CODE, READONLY	
;----------------------------------------------------------------------------
Terminal_setup	PROC				; Serial terminal setup
	EXPORT	Terminal_setup
	push	{LR, R0-R2}			; save used regs	
	OORW	1<<4, CMU_HFPERCLKEN0, CMU	; enable clock to UART0
	OORW	1<<3, CMU_HFBUSCLKEN0		; enable clock to GPIO peripheral
	PUTW	0x129<<3, USARTn_CLKDIV, USART0	; set 115200 bd @ 19 MHz rate
	PUTW	3, USARTn_ROUTEPEN		; enable TX/RX pins at location 0
	OORW	0x14, GPIO_PA_MODEL, GPIO	; configure PA.0/PA.1 for TX/RX
	PUTW	5, USARTn_CMD, USART0		; enable TX, RX
	PUTW	4, USARTn_IEN			; enable RX data valid interrupt
	pop	{PC, R0-R2}			; restore regs and exit
	ENDP
		
;----------------------------------------------------------------------------
WriteChar	PROC				; outputs character in R0
	EXPORT	WriteChar	
	push	{LR, R1, R2}			; save used regs			
	ldr	R1, =USART0
wcl	ldrb	R2, [R1, #USARTn_IF]		; wait for TXBL bit to set
	lsrs	R2, #2
	bcc	wcl
	strb	R0, [R1, #USARTn_TXDATA]	; load char to TX buffer
	pop	{PC, R1, R2}			; restore regs and exit
	ENDP
		
;----------------------------------------------------------------------------
NewLine		PROC				; start new line
	EXPORT	NewLine
	push	{LR, R0}
	movs	R0, #13				; output CR character
	bl	WriteChar
	movs	R0, #10				; output LF character
	bl	WriteChar
	pop	{PC, R0}
	ENDP
				
;----------------------------------------------------------------------------
WriteString	PROC				; outputs 0-terminated string in [R0]
	EXPORT	WriteString
	push	{LR, R0, R1}	
	mov	R1, R0				; R1 points to string
wsl1	ldrb	R0, [R1]			; R0 = next char
	orrs	R0, R0				; end of string?
	beq	wsl2				; YES - exit
	bl	WriteChar			; NO  - output it
	adds	R1, #1				; update char pointer
	b	wsl1
wsl2	pop	{PC, R0, R1}
	ENDP

;----------------------------------------------------------------------------
WriteDec	PROC				; outputs 32-bit unsigned number in R0
	EXPORT	WriteDec
	push	{LR, R0-R3}
	movs	R2, #10				; base of system
	movs	R3, #0				; digits counter
wdl1	udiv	R1, R0, R2			; R1 = R0/10
	mls	R0, R1, R2, R0			; R0 = R0 - R1*10
	push	{R0}				; save digit in stack
	adds	R3, #1				; update digits counter	
	orrs	R0, R1, R1			; R0 = R1
	bne	wdl1
	
wdl2	pop	{R0}
	add	R0, #'0'			; ASCII conversion
	bl	WriteChar
	subs	R3, #1
	bne	wdl2
	pop	{PC, R0-R3}
	ENDP

;----------------------------------------------------------------------------
WriteInt	PROC				; outputs 32-bit signed number in R0
	EXPORT	WriteInt
	push	{LR}	
	orrs	R0, R0				; R0 >= 0?
	bpl	wil1				; YES - proceed
	neg	R0, R0				; NO  - negate R0
	push	{R0}				; and 
	movs	R0, #'-'			; print the '-' sign
	bl	WriteChar
	pop	{R0}
	
wil1	bl	WriteDec			; print |R0|	
	pop	{PC}	
	ENDP

;----------------------------------------------------------------------------
WriteHex	PROC				; outputs 32-bit value in R0 in hexadecimal
	EXPORT	WriteHex
	push	{LR, R0-R2}
	mov	R1, R0
	movs	R2, #8				; # of digits to print
whl1	ror	R1, R1, #28	
	and	R0, R1, #0xF			; chop off a hex digit
	adds	R0, #'0'			; ASCII conversion
	cmp	R0, #58
	blt	whl2
	adds	R0, #7				; extra ASCII conversion
whl2	bl	WriteChar			; print hex digit	
	subs	R2, #1				; update digits counter
	bne	whl1
	pop 	{PC, R0-R2}
	ENDP

;============================================================================
ReadChar	PROC				; returns char in R0
	EXPORT	ReadChar
	push	{LR, R1, R2}
	ldr	R1, =USART0
rcl	ldrb	R2, [R1, #USARTn_IF]		; wait for RXDATAV bit to set
	lsrs	R2, #3
	bcc	rcl
	ldrb	R0, [R1, #USARTn_RXDATA]	; load received char
	cmp	R0, #13
	beq	rc2
	cmp	R0, #10
	beq	rc2
	strb	R0, [R1, #USARTn_TXDATA]	; echo char to terminal
rc2	pop	{PC, R1, R2}	
	ENDP

;----------------------------------------------------------------------------
ReadString	PROC				; R0: buffer, R1=max length, R2=str. length 
	EXPORT	ReadString
	push	{LR, R0, R3}
	mov	R3, R0				; R3 = string buffer address
	movs	R2, #0				; input string length
rsl1	bl	ReadChar	
	cmp	R0, #13				; CR ?
	beq	rsl2
	cmp	R0, #10				; end of input?
	beq	rsl2
	strb	R0, [R3, R2]			; save received char
	adds	R2, #1				; update string length
	cmp	R2, R1				; max lengths reached?
	bne	rsl1				; NO - wait for new char	
	
rsl2	bl	NewLine
	movs	R1, #0				; add terminating 0
	strb	R1, [R3, R2]	
	pop	{PC, R0, R3}
	ENDP
		
;----------------------------------------------------------------------------
ReadInt		PROC				; returns signed number in R0
	EXPORT	ReadInt
	push	{LR, R1, R2, R3}		
	movs	R1, #0				; init number	
	movs	R3, #0				; R3=0 for >= numbers and 1 otherwise
	
ril1	bl	ReadChar			; R0 = digit
	cmp	R0, #13				; end of input?
	beq	ril3
	cmp	R0, #10				; end of input?
	beq	ril3
	cmp	R0, #'-'
	bne	ril2
	movs	R3, #1				; remember '-' sign
	b	ril1	
ril2	cmp	R0, #'0'
	blt	ril1				; ignore wrong char
	cmp	R0, #'9'
	bgt	ril1				; ignore wrong char
	subs	R0, #'0'			; ASCII conversion
	movs	R2, #10				; base system
	mla	R2, R1, R2, R0
	orrs 	R2, R2				; overflow?
	bmi	ril1				; YES - ignore the last digit	
	mov	R1, R2				; NO  - accept the result 
	b	ril1
ril3	bl	NewLine
	mov	R0, R1				; copy the result
	orrs	R3, R3				; R3=0?
	beq	ril4	
	neg	R0, R0				; negate the number otherwise
ril4	pop	{PC, R1, R2, R3}	
	ENDP

;----------------------------------------------------------------------------
ReadHex		PROC				; returns unsigned hex number in R0
	EXPORT	ReadHex
	push	{LR, R1, R2, R3}
	movs	R1, #0				; init number	
	movs	R3, #8				; digits counter 	
	
rhl1	bl	ReadChar			; R0 = digit
	cmp	R0, #13				; end of input?
	beq	rhl6
	cmp	R0, #10				; end of input?
	beq	rhl6
	cmp	R0, #'0'
	blt	rhl1				; ignore wrong char
	cmp	R0, #'9'
	bgt	rhl2				; ignore wrong char
	subs	R0, #'0'			; ASCII conversion
	b	rhl4
	
rhl2	cmp	R0, #'A'
	blt	rhl1				; ignore wrong char
	cmp	R0, #'F'
	bgt	rhl3
	subs	R0, #('A'-10)			; ASCII conversion
	b	rhl4	
	
rhl3	cmp	R0, #'a'
	blt	rhl1				; ignore wrong char
	cmp	R0, #'f'
	bgt	rhl1				; ignore wrong char
	subs	R0, #('a'-10)			; ASCII conversion
rhl4	movs	R2, #16				; base system
	mla	R1, R1, R2, R0
	subs	R3, #1				; update digits counter
	bne	rhl1
	
rhl5	bl	ReadChar
	cmp	R0, #13				; wait for end of input
	bne	rhl5
	
rhl6	bl	NewLine
	mov	R0, R1				; copy the result	
	pop	{PC, R1, R2, R3}
	ENDP

;----------------------------------------------------------------------------

		
	ALIGN
	END